require "behaviours/chaseandattack"
require "behaviours/leash"
require "behaviours/wander"
require "behaviours/faceentity"

local LEASH_RETURN_DIST = 10
local LEASH_MAX_DIST = 30

local MIN_FOLLOW_DIST = 2
local TARGET_FOLLOW_DIST = 2
local MAX_FOLLOW_DIST = 5

local TRADE_DIST = 20

local KEEP_FACE_DIST = 8
local START_FACE_DIST = 6

local MAX_WANDER_DIST = 8

local MAX_CHASE_DIST = 20
local MAX_CHASE_TIME = 10

local fungusred1_brain = Class(Brain, function(self, inst)
    Brain._ctor(self, inst)
end)

local function GetFaceTargetFn(inst)
    local target = GetClosestInstWithTag("player", inst, START_FACE_DIST)
    if target and not target:HasTag("notarget") then
        return target
    end
end

local function KeepFaceTargetFn(inst, target)
    return inst:IsNear(target, KEEP_FACE_DIST) and not target:HasTag("notarget")
end

local function GetTraderFn(inst)
    return FindEntity(inst, TRADE_DIST, function(target) return inst.components.trader:IsTryingToTradeWithMe(target) end, {"player"})
end

local function KeepTraderFn(inst, target)
    return inst.components.trader:IsTryingToTradeWithMe(target)
end

local function HasValidHome(inst)
    return inst.components.homeseeker and 
       inst.components.homeseeker.home and 
       inst.components.homeseeker.home:IsValid()
end

local function GoHomeAction(inst)
    if not inst.components.follower.leader and
        HasValidHome(inst) and
        not inst.components.combat.target then
            return BufferedAction(inst, inst.components.homeseeker.home, ACTIONS.GOHOME)
    end
end

local function GetLeader(inst)
    return inst.components.follower.leader 
end

local function GetHomePos(inst)
    return HasValidHome(inst) and inst.components.homeseeker:GetHomePos()
end

local function GetNoLeaderHomePos(inst)
    if GetLeader(inst) then
        return nil
    end
    return GetHomePos(inst)
end

local function GetFaceTargetFn(inst)
    return inst.components.follower.leader
end

local function KeepFaceTargetFn(inst, target)
    return inst.components.follower.leader == target
end

function fungusred1_brain:OnStart()
local clock = GetClock()
 local day = WhileNode( function() return clock and clock:IsDay() end, "IsDay",
        PriorityNode{ 

        Wander(self.inst, GetNoLeaderHomePos, MAX_WANDER_DIST),


        },.5)


  local night = WhileNode( function() return clock and clock:IsNight() end, "IsNight",
        PriorityNode{

        DoAction(self.inst, GoHomeAction, "go home", true ),

        },.1)

  local dusk = WhileNode( function() return clock and clock:IsDusk() end, "IsDusk",
        PriorityNode{

        Wander(self.inst, GetNoLeaderHomePos, MAX_WANDER_DIST),

        },.1)

    local root = PriorityNode(
    {

    ChaseAndAttack  (self.inst, MAX_CHASE_TIME, MAX_CHASE_DIST),
    Follow          (self.inst, function(inst) return inst.components.follower.leader end , MIN_FOLLOW_DIST, TARGET_FOLLOW_DIST, MAX_FOLLOW_DIST),
    day,
    night,
    dusk
    }, .5)
    
    self.bt = BT(self.inst, root)
    
end

return fungusred1_brain