require "brains/fungusred1_brain"
require "stategraphs/SGfungusred"

local assets=
{
   
    Asset("ANIM", "anim/fungusred.zip"),
    Asset("SOUNDPACKAGE", "sound/fungus.fev"),
    Asset("SOUND", "sound/fungus.fsb"),
}

local prefabs =
{
    "red_cap",
    "gross",
}

SetSharedLootTable( 'fungusredloot',
{
    {'red_cap',  0.1},

})

local function Retarget(inst)
    return FindEntity(inst, TUNING.PIG_TARGET_DIST,
        function(guy)
        if guy.components.combat and guy.components.health and not guy.components.health:IsDead() then
                return guy:HasTag("") and guy.components.health and not guy.components.health:IsDead() and inst.components.combat:CanTarget(guy) and not 
                (inst.components.follower.leader ~= nil and guy:HasTag("abigail"))
            end
        end)
end
local function KeepTarget(inst, target)
    --give up on dead guys, or guys in the dark, or werepigs
    return inst.components.combat:CanTarget(target)
           and (not target.LightWatcher or target.LightWatcher:IsInLight())
           and not (target.sg and target.sg:HasStateTag("transform") )
end

local function OnAttacked(inst, data)
    inst.components.combat:SetTarget(data.attacker)
    inst.components.combat:ShareTarget(data.attacker, 30, function(dude)
        return dude:HasTag("fungus")
               and not dude.components.health:IsDead()
               and dude.components.follower
               and dude.components.follower.leader == inst.components.follower.leader
    end, 10)
end


 local function ShouldAcceptItem(inst, item)
    if inst.components.health and inst.components.health:IsDead() then return false end
    if item:HasTag("gross")  then
        return true
    else
        return false
    end
end

function GetOtherFungus(inst)
    local x,y,z = inst.Transform:GetWorldPosition()
    local ents = TheSim:FindEntities(x,y,z, 100,  {"fungus"}, {"FX", "NOCLICK", "DECOR","INLIMBO"})
    return ents
end

local function OnGetItemFromPlayer(inst, giver, item)
    if item:HasTag("gross")  then 
        local playedfriendsfx = false
        if inst.components.combat.target and inst.components.combat.target == giver then
            inst.components.combat:SetTarget(nil)
        elseif giver.components.leader then
            inst.SoundEmitter:PlaySound("dontstarve/common/makeFriend")
            playedfriendsfx = true
            giver.components.leader:AddFollower(inst)
            local loyaltyTime = 1000000000
            inst.components.follower:AddLoyaltyTime(loyaltyTime)
        end

        local Fungus = GetOtherFungus(inst)
        local maxFungus = 3

        for k,v in pairs(Fungus) do
            if maxFungus < 0 then
                break
            end

            if v.components.combat.target and v.components.combat.target == giver then
                v.components.combat:SetTarget(nil)
            elseif giver.components.leader then
                if not playedfriendsfx then
                    v.SoundEmitter:PlaySound("dontstarve/common/makeFriend")
                    playedfriendsfx = true
                end
                giver.components.leader:AddFollower(v)
                local loyaltyTime = 1000000000
                if v.components.follower then
                    v.components.follower:AddLoyaltyTime(loyaltyTime)
                end
            end
            maxFungus = maxFungus - 1
        end
    end
end
local function OnRefuseItem(inst, item)
    inst.sg:GoToState("refuse")
end

local function init_prefab()
    
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()

    inst.entity:AddSoundEmitter()
    inst.Transform:SetFourFaced()      
inst:AddTag("fungus")

anim:SetBank("fungusred")

anim:SetBuild("fungusred")

inst:AddComponent("inspectable")

inst:AddComponent("trader")
    inst.components.trader:SetAcceptTest(ShouldAcceptItem)
    inst.components.trader.onaccept = OnGetItemFromPlayer
    inst.components.trader.onrefuse = OnRefuseItem
 
inst:AddComponent("locomotor")
    inst.components.locomotor.runspeed = 3
    MakeCharacterPhysics(inst, 10, 0.25)
   
    inst:AddComponent("follower")
    inst.components.follower.maxfollowtime = TUNING.TOTAL_DAY_TIME

inst:AddComponent("combat")
    inst.components.combat.hiteffectsymbol = "body"
    inst.components.combat:SetTarget(nil)
    inst.components.combat:SetDefaultDamage(10)
    inst.components.combat:SetAttackPeriod(3)
    inst.components.combat:SetRetargetFunction(3, Retarget)
    inst.components.combat:SetKeepTargetFunction(KeepTarget)
    inst.components.combat:SetRange(1)    

inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetChanceLootTable('fungusredloot')

inst:AddComponent("knownlocations")


inst:AddComponent("health")
    inst.components.health:SetMaxHealth(75)
    
inst:ListenForEvent("attacked", OnAttacked)

inst:SetStateGraph("SGfungusred")
    
local brain = require "brains/fungusred1_brain"
inst:SetBrain(brain)
return inst
end

return Prefab( "fungusred", init_prefab, assets, nil)
