require "prefabutil"
require "recipes"

local assets =
{
    Asset("ANIM", "anim/fungusredhouse.zip"),
    Asset("SOUND", "sound/pig.fsb"),
    Asset("ATLAS", "images/inventoryimages/fungusredhouse.xml"),
    Asset("IMAGE", "images/inventoryimages/fungusredhouse.tex"),    
}

local prefabs = 
{
    "fungusred",
}

SetSharedLootTable( 'houseloot',
{
    {'red_cap', 1.0},
    {'log',     1.0},
    {'log',     1.0},
    {'red_cap', 0.5},
})
local function onhammered(inst, worker)
    inst:RemoveComponent("childspawner")
    inst.components.lootdropper:DropLoot()
    SpawnPrefab("collapse_big").Transform:SetPosition(inst.Transform:GetWorldPosition())
    inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
    inst:Remove()
end

local function onhit(inst, worker)
    if inst.components.childspawner then
        inst.components.childspawner:ReleaseAllChildren(worker)
    end
    inst.AnimState:PlayAnimation("hit")
    inst.AnimState:PushAnimation("idle")
end

local function StartSpawning(inst)
    if inst.components.childspawner and GetSeasonManager() and not GetSeasonManager():IsWinter() then
        inst.components.childspawner:StartSpawning()
    end
end

local function StopSpawning(inst)
    if inst.components.childspawner then
        inst.components.childspawner:StopSpawning()
    end
end

local function OnSpawned(inst, child)
    inst.SoundEmitter:PlaySound("dontstarve/common/pighouse_door")
    if GetClock():IsDay() and inst.components.childspawner and inst.components.childspawner:CountChildrenOutside() >= 1 then
        StartSpawning(inst)
    end
end

local function OnGoHome(inst, child) 
    inst.SoundEmitter:PlaySound("dontstarve/common/pighouse_door")
    if inst.components.childspawner and inst.components.childspawner:CountChildrenOutside() < 1 then
        StartSpawning(inst)
    end
end

local function onbuilt(inst)
    inst.AnimState:PlayAnimation("place")
    inst.AnimState:PushAnimation("idle")
end

local function fn()
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()

    local minimap = inst.entity:AddMiniMapEntity()
    minimap:SetIcon( "fungusredhouse.tex" )
    
    MakeObstaclePhysics(inst, 0.5)

    anim:SetBank("fungusredhouse")
    anim:SetBuild("fungusredhouse")
    anim:PlayAnimation("idle")

    inst:AddTag("structure")

    inst:AddComponent("lootdropper")

    inst.components.lootdropper:SetChanceLootTable('houseloot')

    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
    inst.components.workable:SetWorkLeft(2)
    inst.components.workable:SetOnFinishCallback(onhammered)
    inst.components.workable:SetOnWorkCallback(onhit)
    
    inst:AddComponent("childspawner")
    inst.components.childspawner.childname = "fungusred"
    inst.components.childspawner:SetSpawnedFn(OnSpawned)
    inst.components.childspawner:SetGoHomeFn(OnGoHome)
    inst.components.childspawner:SetRegenPeriod(TUNING.SPIDERDEN_RELEASE_TIME)
    inst.components.childspawner:SetSpawnPeriod(TUNING.SPIDERDEN_RELEASE_TIME)
    inst.components.childspawner:SetMaxChildren(4)

        inst:ListenForEvent("daytime", function() 
        if GetSeasonManager() and not GetSeasonManager():IsWinter() then
            inst.components.childspawner:ReleaseAllChildren()
        end
        StartSpawning(inst)
    end, GetWorld())

    inst:ListenForEvent("daytime", function() StartSpawning(inst) end , GetWorld())
    inst:ListenForEvent("dusktime", function() StopSpawning(inst) end , GetWorld())
    inst:ListenForEvent("nighttime", function() StopSpawning(inst) end , GetWorld())

 inst:ListenForEvent( "onbuilt", function()
        anim:PlayAnimation("build")
        anim:PushAnimation("idle")

    end)

    inst:AddComponent("inspectable")
    
    MakeSnowCovered(inst, 1)
    return inst
end

return Prefab( "fungusredhouse", fn, assets, prefabs ),
       MakePlacer("common/fungusred_placer", "fungusredhouse", "fungusredhouse", "idle")