require "brains/wolphin_brain"
require "stategraphs/SGwolphin"

local assets=
{
    Asset("ANIM", "anim/wolphin.zip"),
    Asset("SOUNDPACKAGE", "sound/fungus.fev"),
    Asset("SOUND", "sound/fungus.fsb"),
}

local prefabs =
{
    "blue_cap",
    "gross",
}

SetSharedLootTable( 'wolphinloot',
{
    {'blue_cap',  0.1},

})

local function Retarget(inst)
    return FindEntity(inst, TUNING.PIG_TARGET_DIST,
        function(guy)
            if not guy.LightWatcher or guy.LightWatcher:IsInLight() then
                return guy:HasTag("") and guy.components.health and not guy.components.health:IsDead() and inst.components.combat:CanTarget(guy) and not 
            (inst.components.follower.leader ~= nil and guy:HasTag("abigail"))
        end
    end)
end
local function KeepTarget(inst, target)
    return inst.components.combat:CanTarget(target)
           and (not target.LightWatcher or target.LightWatcher:IsInLight())
        and not (target.sg and target.sg:HasStateTag("transform") )
    end

local function OnAttacked(inst, data)
    inst.components.combat:SetTarget(data.attacker)
    inst.components.combat:ShareTarget(data.attacker, 30, function(dude)
        return dude:HasTag("fungus")
               and not dude.components.health:IsDead()
               and dude.components.follower
               and dude.components.follower.leader == inst.components.follower.leader
    end, 10)
end


local function ShouldAcceptItem(inst, item)
    if inst.components.health and inst.components.health:IsDead() then return false end
    if item:HasTag("gross")  then
        return true
    else
        return false
    end
end

local function OnGetItemFromPlayer(inst, giver, item)
        if item:HasTag("gross") then
            if inst.components.combat.target and inst.components.combat.target == giver then
                inst.components.combat:SetTarget(nil)
            elseif giver.components.leader then
                giver.components.leader:AddFollower(inst)
                inst.components.follower:AddLoyaltyTime(TUNING.PIG_LOYALTY_MAXTIME+TUNING.PIG_LOYALTY_MAXTIME)
            end
        end
    end

local function OnRefuseItem(inst, item)
    inst.sg:GoToState("refuse")
end

local function init_prefab()
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    
inst.entity:AddSoundEmitter()   
inst.Transform:SetTwoFaced()

inst:AddTag("fungus")

anim:SetBank("wolphin")
anim:SetBuild("wolphin")

inst:AddComponent("inspectable")
inst:AddComponent("trader")
    inst.components.trader:SetAcceptTest(ShouldAcceptItem)
    inst.components.trader.onaccept = OnGetItemFromPlayer
    inst.components.trader.onrefuse = OnRefuseItem
inst:AddComponent("locomotor")
    inst.components.locomotor.runspeed = 2
    MakeCharacterPhysics(inst, 10, 0.25)
inst:AddComponent("follower")
    inst.components.follower.maxfollowtime = TUNING.PIG_LOYALTY_MAXTIME+TUNING.PIG_LOYALTY_MAXTIME
inst:AddComponent("combat")
    inst.components.combat.hiteffectsymbol = "wolphinbody"
    inst.components.combat:SetDefaultDamage(20)
    inst.components.combat:SetAttackPeriod(1)
    inst.components.combat:SetRetargetFunction(3, Retarget)
    inst.components.combat:SetKeepTargetFunction(KeepTarget)
    inst.components.combat:SetRange(1)
inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetChanceLootTable('wolphinloot')
inst:AddComponent("knownlocations")
inst:AddComponent("health")
    inst.components.health:SetMaxHealth(100)
    
inst:ListenForEvent("attacked", OnAttacked)

inst:SetStateGraph("SGwolphin")
    
local brain = require "brains/wolphin_brain"
    inst:SetBrain(brain)
    return inst
end

return Prefab( "wolphin", init_prefab, assets, nil)
