require("stategraphs/commonstates")

local actionhandlers = 
{
    ActionHandler(ACTIONS.GOHOME, "gohome"),
    ActionHandler(ACTIONS.EAT, "eat"),


}

local events=
{
    CommonHandlers.OnStep(),
    CommonHandlers.OnLocomote(false,true),
    CommonHandlers.OnSleep(),
    CommonHandlers.OnFreeze(),
    CommonHandlers.OnAttack(),
    CommonHandlers.OnAttacked(true),
    CommonHandlers.OnDeath(),
}

local states=
{

    State{

        name = "idle",
        tags = {"idle", "canrotate"},

        onenter = function(inst, playanim)
            inst.AnimState:PlayAnimation("idle", true)
        end,

    },


    State{

        name = "walk",
        tags = {"walk", "canrotate", "moving"},

        onenter = function(inst, cb)
            inst.AnimState:PlayAnimation("walk", true)
            inst.components.locomotor:RunForward()            
        end,
    },

    State{

        name = "walk_start",
        tags = {"walk_start", "canrotate", "moving"},

        onenter = function(inst, cb)
            inst.AnimState:PlayAnimation("walk", true)
            inst.components.locomotor:RunForward()            
        end,
    },

    State{

        name = "walk_stop",
        tags = {"walk_stop", "canrotate", "moving"},

        onenter = function(inst, cb)
            inst.AnimState:PlayAnimation("idle", true)            
        end,

        events=
        {
            EventHandler("animover", function(inst) inst.sg:GoToState("idle") end),
        },
    },

    State{
        name = "attack",
        tags = {"attack"},
        
        onenter = function(inst)
            inst.components.combat:StartAttack()
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("atk")
        end,
        
        timeline=
        {
            TimeEvent(13*FRAMES, function(inst) inst.components.combat:DoAttack() inst.sg:RemoveStateTag("attack") inst.sg:RemoveStateTag("busy") end),
            TimeEvent(16*FRAMES, function(inst) inst.components.combat:DoAttack() inst.sg:RemoveStateTag("attack") inst.sg:RemoveStateTag("busy") end),         
        },
        
        events=
        {
            EventHandler("animqueueover", function(inst) inst.sg:GoToState("idle") end),
        },
    },

State{
        name = "death",
        tags = {"death","busy"},
        
        onenter = function(inst)
            inst.AnimState:PlayAnimation("dead")
            inst.SoundEmitter:PlaySound("fungus/sound/dead")
            inst.Physics:Stop()
            RemovePhysicsColliders(inst)            
            inst.components.lootdropper:DropLoot(Vector3(inst.Transform:GetWorldPosition()))            
        end,
   
    },

     State{
        name = "hit",
        tags = {"busy"},

        onenter = function(inst)
            inst.SoundEmitter:PlaySound("fungus/sound/hit")
            inst.AnimState:PlayAnimation("hit")
            inst.Physics:Stop()
        end,

        events=
        {
            EventHandler("animover", function(inst) inst.sg:GoToState("idle") end ),
        },
    },

}


CommonStates.AddSimpleState(states,"refuse", "reject", {"busy"})

CommonStates.AddSimpleActionState(states, "gohome", "pig_pickup", 4*FRAMES, {"busy"})

return StateGraph("SGfungusgreen", states, events, "idle", actionhandlers)