require("stategraphs/commonstates")

local states=
{

    State{

        name = "idle",
        tags = {"idle", "canrotate"},

        onenter = function(inst, playanim)
            inst.AnimState:PlayAnimation("idle", true)
        end,

    },

State{
        name = "hatch",
        tags = {"busy"},
        onenter = function(inst)
            inst.AnimState:PlayAnimation("hatch_stop")
        end,
        events=
        {
            EventHandler("animover", function(inst)
                inst.sg:GoToState("idle")
            end),
        },
    },

    State{

        name = "walk",
        tags = {"walk", "canrotate", "moving"},

        onenter = function(inst, playanim)
            inst.AnimState:PlayAnimation("walk", true)
            inst.components.locomotor:RunForward()            
        end,
    },    

    State{
        name = "attack",
        tags = {"attack", "busy"},
        
        onenter = function(inst)
            inst.components.combat:StartAttack()
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("atk")
        end,
        
        timeline=
        {
            TimeEvent(20*FRAMES, function(inst) inst.components.combat:DoAttack() inst.sg:RemoveStateTag("attack") inst.sg:RemoveStateTag("busy") end),
            
        },
        
        events=
        {
            EventHandler("animqueueover", function(inst) inst.sg:GoToState("idle") end),
        },
    },

State{
        name = "death",
        tags = {"busy"},
        
        onenter = function(inst)
            inst.AnimState:PlayAnimation("dead")
            inst.Physics:Stop()
            RemovePhysicsColliders(inst)            
            inst.components.lootdropper:DropLoot(Vector3(inst.Transform:GetWorldPosition()))            
        end,
        
    },

     State{
        name = "hit",
        tags = {"busy"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("hit")
            inst.Physics:Stop()
        end,

        events=
        {
            EventHandler("animover", function(inst) inst.sg:GoToState("idle") end ),
        },
    },

}

local event_handlers=
{
EventHandler("death", function(inst) inst.sg:GoToState("death") end),
    EventHandler("doattack", function(inst) if inst.components.health:GetPercent() > 0 and not inst.sg:HasStateTag("busy") then inst.sg:GoToState("attack") end end),

    EventHandler("locomote", 
        function(inst) 
            if not inst.sg:HasStateTag("idle") and not inst.sg:HasStateTag("moving") then return end

            if inst.components.locomotor:WantsToMoveForward() then

                if not inst.sg:HasStateTag("moving") then
                    inst.sg:GoToState("walk")
                end

            else

                if not inst.sg:HasStateTag("idle") then
                    inst.sg:GoToState("idle")
                end

            end
        end),
}

CommonStates.AddSimpleState(states,"refuse", "reject", {"busy"})

return StateGraph("SGowlchick", states, event_handlers, "idle", {})