require("stategraphs/commonstates")

local actionhandlers = 
{
    ActionHandler(ACTIONS.GOHOME, "gohome"),
    ActionHandler(ACTIONS.EAT, "eat"),


}

local events=
{
    CommonHandlers.OnStep(),
    CommonHandlers.OnLocomote(false,true),
    CommonHandlers.OnSleep(),
    CommonHandlers.OnFreeze(),
    CommonHandlers.OnAttack(),
    CommonHandlers.OnAttacked(true),
    CommonHandlers.OnDeath(),
}

local states=
{

    State{

        name = "idle",
        tags = {"idle", "canrotate"},

        onenter = function(inst, playanim)
            inst.AnimState:PlayAnimation("idle", true)
        end,

    },


    State{

        name = "walk",
        tags = {"walk", "canrotate", "moving"},

        onenter = function(inst, cb)
            inst.AnimState:PlayAnimation("walk", true)
            inst.components.locomotor:RunForward()            
        end,
    },

    State{

        name = "walk_start",
        tags = {"walk_start", "canrotate", "moving"},

        onenter = function(inst, cb)
            inst.AnimState:PlayAnimation("walk", true)
            inst.components.locomotor:RunForward()            
        end,
    },

    State{

        name = "walk_stop",
        tags = {"walk_stop", "canrotate", "moving"},

        onenter = function(inst, cb)
            inst.AnimState:PlayAnimation("idle", true)            
        end,

        events=
        {
            EventHandler("animover", function(inst) inst.sg:GoToState("idle") end),
        },
    },



    State{
        name = "summon",
        tags = {"summon", "busy"},

        onenter = function(inst, playanim)
            inst.Physics:Stop()
            inst.components.locomotor:Stop()
            local angle = TheCamera:GetHeadingTarget()*DEGREES -- -22.5*DEGREES
            inst.Transform:SetRotation(angle / DEGREES)
            inst.AnimState:PlayAnimation("summon")

        end,

        timeline=
        {
            TimeEvent(17*FRAMES, function(inst) inst.SoundEmitter:PlaySound("puppeteer/sound/summon") end),
            TimeEvent(30*FRAMES, function(inst)
            
                local angle = inst.Transform:GetRotation()/DEGREES
                local prefab = (inst.components.combat.target and math.random() < .333) and "evilpuppet" or "evilpuppet"
                local spider = inst.components.lootdropper:SpawnLootPrefab(prefab)
                local rad = spider.Physics:GetRadius()+inst.Physics:GetRadius()+.25;
                local pt = Vector3(inst.Transform:GetWorldPosition())
                if spider then
                    spider.Transform:SetPosition(pt.x + rad*math.cos(angle), pt.y, pt.z + rad*math.sin(angle))
                    spider.sg:GoToState("spawn")
                    inst.components.leader:AddFollower(spider)
                    if inst.components.combat.target then
                        spider.components.combat:SetTarget(inst.components.combat.target)
                    end
                end
            end),
        },

        events=
        {
            EventHandler("animover", function(inst) inst.sg:GoToState("idle") end),
        },
    },    

    State{
        name = "attack",
        tags = {"attack"},
        
        onenter = function(inst)
            inst.components.combat:StartAttack()
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("atk")
        end,
        
        timeline=
        {
            TimeEvent(17*FRAMES, function(inst) inst.components.combat:DoAttack() inst.sg:RemoveStateTag("attack")end),
            
        },
        
        events=
        {
            EventHandler("animover", function(inst) inst.sg:GoToState("idle") end),
        },
    },

State{
        name = "death",
        tags = {"death","busy"},
        
        onenter = function(inst)
            inst.AnimState:PlayAnimation("dead")
            inst.Physics:Stop()
            RemovePhysicsColliders(inst)            
            inst.components.lootdropper:DropLoot(Vector3(inst.Transform:GetWorldPosition()))            
        end,
        timeline=
        {
            TimeEvent(13*FRAMES, function(inst) inst.SoundEmitter:PlaySound("puppeteer/sound/dead") end),
        },      
    },

     State{
        name = "hit",
        tags = {"busy"},

        onenter = function(inst)
            inst.SoundEmitter:PlaySound("puppeteer/sound/hit")
            inst.AnimState:PlayAnimation("hit")
            inst.Physics:Stop()
        end,

        events=
        {
            EventHandler("animover", function(inst) inst.sg:GoToState("idle") end),
        },
    },

}

return StateGraph("SGpuppeteer", states, events, "idle", actionhandlers)